<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Api\Data;

interface TransactionInterface extends \Magento\Framework\Api\ExtensibleDataInterface
{
    const ADDED = 'added';
    const ORDER_IDS = 'order_ids';
    const QUOTE_ID = 'quote_id';
    const REQUEST_BODY = 'request_body';
    const REQUEST_HEADERS = 'request_headers';
    const REQUEST_METHOD = 'request_method';
    const REQUEST_URI = 'request_uri';
    const RESPONSE_BODY = 'response_body';
    const RESPONSE_HEADERS = 'response_headers';
    const RESPONSE_STATUS_CODE = 'response_status_code';
    const SHIPMENT_IDS = 'shipment_ids';
    const TRANSACTION_ID = 'transaction_id';

    /**
     * Get added
     *
     * @return null|string
     */
    public function getAdded(): ?string;

    /**
     * Get extension attributes
     *
     * @return \Mediastrategi\PostNord\Api\Data\TransactionExtensionInterface|null
     */
    public function getExtensionAttributes();

    /**
     * Get order ids
     *
     * @return null|string
     */
    public function getOrderIds(): ?string;

    /**
     * Get quote id
     *
     * @return null|string
     */
    public function getQuoteId(): ?string;

    /**
     * Get request body
     *
     * @return null|string
     */
    public function getRequestBody(): ?string;

    /**
     * Get request headers
     *
     * @return null|string
     */
    public function getRequestHeaders(): ?string;

    /**
     * Get request method
     *
     * @return null|string
     */
    public function getRequestMethod(): ?string;

    /**
     * Get request uri
     *
     * @return null|string
     */
    public function getRequestUri(): ?string;

    /**
     * Get response body
     *
     * @return null|string
     */
    public function getResponseBody(): ?string;

    /**
     * Get response headers
     *
     * @return null|string
     */
    public function getResponseHeaders(): ?string;

    /**
     * Get response status code
     *
     * @return null|int
     */
    public function getResponseStatusCode(): ?int;

    /**
     * Get shipment ids
     *
     * @return null|string
     */
    public function getShipmentIds(): ?string;

    /**
     * Get transaction id
     *
     * @return null|int
     */
    public function getTransactionId(): ?int;

    /**
     * Set added
     *
     * @param string $added
     *
     * @return TransactionInterface
     */
    public function setAdded(string $added): TransactionInterface;

    /**
     * Set extension attributes
     *
     * @param \Mediastrategi\PostNord\Api\Data\TransactionExtensionInterface $extensionAttributes
     *
     * @return TransactionInterface
     */
    public function setExtensionAttributes(
        \Mediastrategi\PostNord\Api\Data\TransactionExtensionInterface $extensionAttributes
    ): TransactionInterface;

    /**
     * Set order ids
     *
     * @param string $orderIds
     *
     * @return TransactionInterface
     */
    public function setOrderIds(string $orderIds): TransactionInterface;

    /**
     * Set quote id
     *
     * @param string $quoteId
     *
     * @return TransactionInterface
     */
    public function setQuoteId(string $quoteId): TransactionInterface;

    /**
     * Set request body
     *
     * @param string $requestBody
     *
     * @return TransactionInterface
     */
    public function setRequestBody(string $requestBody): TransactionInterface;

    /**
     * Set request headers
     *
     * @param string $requestheaders
     *
     * @return TransactionInterface
     */
    public function setRequestHeaders(string $requestHeaders): TransactionInterface;

    /**
     * Set request method
     *
     * @param string $requestMethod
     *
     * @return TransactionInterface
     */
    public function setRequestMethod(string $requestMethod): TransactionInterface;

    /**
     * Set request uri
     *
     * @param string $requestUri
     *
     * @return TransactionInterface
     */
    public function setRequestUri(string $requestUri): TransactionInterface;

    /**
     * Set response body
     *
     * @param string $responseBody
     *
     * @return TransactionInterface
     */
    public function setResponseBody(string $responseBody): TransactionInterface;

    /**
     * Set response headers
     *
     * @param string $responseHeaders
     *
     * @return TransactionInterface
     */
    public function setResponseHeaders(string $responseHeaders): TransactionInterface;

    /**
     * Set response status code
     *
     * @param int $responseStatusCode
     *
     * @return TransactionInterface
     */
    public function setResponseStatusCode(int $responseStatusCode): TransactionInterface;

    /**
     * Set shipment ids
     *
     * @param string $shipmentIds
     *
     * @return TransactionInterface
     */
    public function setShipmentIds(string $shipmentIds): TransactionInterface;

    /**
     * Set transaction id
     *
     * @param int $transactionId
     *
     * @return TransactionInterface
     */
    public function setTransactionId(int $transactionId): TransactionInterface;
}
