<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Api;

interface TransactionRepositoryInterface
{
    /**
     * Save Transaction
     *
     * @param  \Mediastrategi\PostNord\Api\Data\TransactionInterface  $transaction
     *
     * @return \Mediastrategi\PostNord\Api\Data\TransactionInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function save(
        \Mediastrategi\PostNord\Api\Data\TransactionInterface $transaction
    );

    /**
     * Retrieve Transaction
     *
     * @param  string  $transactionId
     *
     * @return \Mediastrategi\PostNord\Api\Data\TransactionInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function get($transactionId);

    /**
     * Retrieve Transaction matching the specified criteria.
     *
     * @param  \Magento\Framework\Api\SearchCriteriaInterface  $searchCriteria
     *
     * @return \Mediastrategi\PostNord\Api\Data\TransactionSearchResultsInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
    );

    /**
     * Delete Transaction
     *
     * @param  \Mediastrategi\PostNord\Api\Data\TransactionInterface  $transaction
     *
     * @return bool true on success
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function delete(
        \Mediastrategi\PostNord\Api\Data\TransactionInterface $transaction
    );

    /**
     * Delete Transaction by ID
     *
     * @param  string  $transactionId
     *
     * @return bool true on success
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function deleteById($transactionId);
}
