<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Controller\Adminhtml\Shipment;

class Label extends \Magento\Backend\App\Action implements \Magento\Framework\App\Action\HttpGetActionInterface
{
    /**
     * Authorization level of a basic admin session
     */
    public const ADMIN_RESOURCE = 'Mediastrategi_PostNord::automation';

    /**
     * @var \Magento\Framework\App\Response\Http\FileFactory
     */
    private $fileFactory;

    /**
     * @var \Mediastrategi\PostNord\Helper\Shipment
     */
    private $shipmentHelper;

    /**
     * \Magento\Sales\Api\ShipmentRepositoryInterface
     */
    private $shipmentRepository;

    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\App\Response\Http\FileFactory $fileFactory,
        \Mediastrategi\PostNord\Helper\Shipment $shipmentHelper,
        \Magento\Sales\Api\ShipmentRepositoryInterface $shipmentRepository
    ) {
        parent::__construct($context);
        $this->fileFactory = $fileFactory;
        $this->shipmentHelper = $shipmentHelper;
        $this->shipmentRepository = $shipmentRepository;
    }

    public function execute(): \Magento\Framework\App\ResponseInterface
    {
        $resultRedirect =
            $this->resultRedirectFactory->create();
        $orderId =
            (int) $this->getRequest()->getParam('order');
        $shipmentId =
            (string) $this->getRequest()->getParam('id');
        if (
            !empty($orderId)
            && !empty($shipmentId)
        ) {
            $shipment = $this->shipmentHelper->getOrderShipmentFromExternalShipmentId(
                $orderId,
                $shipmentId
            );
            if ($shipment) {
                return $this->fileFactory->create(
                    'Shipping-Label-' . $shipmentId . '.pdf',
                    $shipment->getShippingLabel(),
                    \Magento\Framework\App\Filesystem\DirectoryList::VAR_DIR,
                    'application/pdf'
                );
            } else {
                throw new \Magento\Framework\Exception\LocalizedException(
                    __(
                        'Invalid shipment id! %1',
                        $shipmentId
                    )
                );
            }
        } else {
            throw new \Magento\Framework\Exception\LocalizedException(
                __(
                    'Missing Order Id of Shipment Id!'
                )
            );
        }

        $this->messageManager->addErrorMessage(
            __(
                'Failed to find label for shipment %d',
                $shipmentId
            )
        );
        return $resultRedirect->setPath('sales/order/index');
    }
}
