<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Controller\Checkout;

class Update extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $cartRepoInterface;

    /**
     * @var \Magento\Quote\Api\Data\CartExtensionFactory
     */
    private $cartExtensionFactory;

    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    private $resultJsonFactory;

    /**
     * @var \Magento\Checkout\Model\Cart
     */
    private $cartModel;

    /**
     * @var \Mediastrategi\PostNord\Helper\PersistentQuote
     */
    private $persistentQuoteHelper;

    public function __construct(
        \Magento\Quote\Api\Data\CartExtensionFactory $cartExtensionFactory,
        \Magento\Checkout\Model\Cart $cartModel,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepoInterface,
        \Mediastrategi\PostNord\Helper\Meta $metaHelper,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Mediastrategi\PostNord\Helper\PersistentQuote $persistentQuoteHelper,
        \Magento\Framework\App\Action\Context $context
    ) {
        parent::__construct($context);
        $this->cartExtensionFactory = $cartExtensionFactory;
        $this->cartModel = $cartModel;
        $this->cartRepoInterface = $cartRepoInterface;
        $this->metaHelper = $metaHelper;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->persistentQuoteHelper = $persistentQuoteHelper;
    }

    /**
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function execute(): \Magento\Framework\Controller\Result\Json
    {
        $resultJson =
            $this->resultJsonFactory->create();
        $oldState =
            [];
        $newState =
            [];
        $postState =
            [];
        $quoteId =
            0;
        $stateHasChanged =
            false;

        if ($this->metaHelper->isEnabledForCheckout()) {
            // Get old state
            if (
                !$quote =
                $this->cartModel->getQuote()
            ) {
                throw new \Magento\Framework\Exception\LocalizedException(
                    __(
                        'Failed to get session quote!'
                    )
                );
            }
            $quoteId =
                (int) $quote->getId();
            $quoteData =
                $this->persistentQuoteHelper->readData($quoteId);
            if ($quoteData) {
                $oldState = $quoteData;
                $this->metaHelper->log(
                    (string) __(
                        'Found old quote data %1 for quote with id: %2',
                        var_export($quoteData, true),
                        var_export($oldState, true)
                    )
                );
            } else {
                $this->metaHelper->log(
                    (string) __(
                        'Found no old quote data for quote with id: %1',
                        $quoteId
                    )
                );
            }

            // Get posted state
            $postState =
                $this->getRequest()->getParam('state') ?? [];

            /**
             * If we have a posted state and it differs from the old state
             * save it as a new state on the current quote
             */
            if (
                is_array($postState)
                && !empty($postState)
                && $postState != $oldState
            ) {
                $stateHasChanged =
                    true;
                $newState =
                    $postState;
                $quoteId =
                    (int) $quote->getId();
                if (
                    $this->persistentQuoteHelper->update(
                        $quoteId,
                        $newState
                    )
                ) {
                    $this->metaHelper->log(
                        (string) __(
                            'Saved new quote data %1 on quote with id: %2',
                            var_export($newState, true),
                            $quoteId
                        )
                    );
                } else {
                    throw new \Magento\Framework\Exception\LocalizedException(
                        __(
                            'Failed to update quote data for quote with id: %1',
                            $quoteId
                        )
                    );
                }
            } else {
                $this->metaHelper->log(
                    (string) __(
                        'Posted state is empty or matches old state %1',
                        var_export($postState, true)
                    )
                );
            }
        }

        return $resultJson->setData(
            [
                'old_state' =>
                    $oldState,
                'new_state' =>
                    $newState,
                'post_state' =>
                    $postState,
                'state_has_changed' =>
                    $stateHasChanged,
                'quote_id' =>
                    $quoteId,
            ]
        );
    }
}
