<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Helper;

class OrderData extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var array
     */
    private static $orderDataCache = [];

    /**
     * @var \Mediastrategi\PostNord\Api\Data\OrderDataInterfaceFactory
     */
    private $orderDataFactory;

    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $cartRepoInterface;

    /**
     * @var \Mediastrategi\PostNord\Api\OrderDataRepositoryInterface
     */
    private $orderDataRepository;

    /**
     * @var \Mediastrategi\PostNord\Model\ResourceModel\OrderData\CollectionFactory
     */
    private $orderDataCollectionFactory;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Mediastrategi\PostNord\Api\OrderDataRepositoryInterface $orderDataRepository,
        \Mediastrategi\PostNord\Api\Data\OrderDataInterfaceFactory $orderDataFactory,
        \Mediastrategi\PostNord\Model\ResourceModel\OrderData\CollectionFactory $orderDataCollectionFactory,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepoInterface
    ) {
        parent::__construct($context);
        $this->orderDataRepository =
            $orderDataRepository;
        $this->orderDataFactory =
            $orderDataFactory;
        $this->orderDataCollectionFactory =
            $orderDataCollectionFactory;
        $this->cartRepoInterface =
            $cartRepoInterface;
    }

    public function create(
        int $orderId,
        array $orderData
    ): bool {
        $newOrderData =
            $this->orderDataFactory->create();
        $newOrderData->setOrderId($orderId);
        $newOrderData->setOrderData(json_encode($orderData));
        try {
            $this->orderDataRepository->save($newOrderData);
            return true;
        } catch (\Magento\Framework\Exception\CouldNotSaveException $e) {
            $this->_logger->error(
                __(
                    'Could not save OrderData: %1, Order-Id: %2, Data: %3',
                    $e->getMessage(),
                    var_export($orderId, true),
                    var_export($orderData, true)
                )
            );
            return false;
        }
    }

    public function read(
        int $orderId
    ): ?\Mediastrategi\PostNord\Api\Data\OrderDataInterface {
        $orderDataCollection =
            $this->orderDataCollectionFactory->create();
        $orderData = $orderDataCollection
            ->addFieldToFilter('order_id', $orderId)
            ->getFirstItem();
        if ($orderData->getId()) {
            return $orderData->getDataModel();
        }
        return null;
    }

    public function readData(
        int $orderId
    ): ?array {
        if (
            !array_key_exists(
                $orderId,
                self::$orderDataCache
            )
        ) {
            $return = null;
            if ($object = $this->read($orderId)) {
                try {
                    $data = json_decode(
                        $object->getOrderData(),
                        true
                    );
                    if (!empty($data)) {
                        $return = $data;
                    }
                } catch (\Exception $e) {
                    $return = null;
                }
            }
            self::$orderDataCache[$orderId] = $return;
        }
        return self::$orderDataCache[$orderId];
    }

    public function update(
        int $orderId,
        array $orderData
    ): bool {
        if ($object = $this->read($orderId)) {
            $object->setOrderData(json_encode($orderData));
            $this->orderDataRepository->save($object);
            if (
                array_key_exists(
                    $orderId,
                    self::$orderDataCache
                )
            ) {
                unset(self::$orderDataCache[$orderId]);
            }
            return true;
        } else {
            if (
                $this->create(
                    $orderId,
                    $orderData
                )
            ) {
                if (
                    array_key_exists(
                        $orderId,
                        self::$orderDataCache
                    )
                ) {
                    unset(self::$orderDataCache[$orderId]);
                }
                return true;
            }
        }
        return false;
    }

    public function delete(int $orderId): bool
    {
        if ($object = $this->read($orderId)) {
            $this->orderDataRepository->delete($object);
            if (
                array_key_exists(
                    $orderId,
                    self::$orderDataCache
                )
            ) {
                unset(self::$orderDataCache[$orderId]);
            }
            return true;
        }
        return false;
    }
}
