<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Helper;

class PersistentQuote extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterfaceFactory
     */
    private $persistentQuoteFactory;

    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $cartRepoInterface;

    /**
     * @var \Mediastrategi\PostNord\Api\PersistentQuoteRepositoryInterface
     */
    private $persistentQuoteRepository;

    /**
     * @var \Mediastrategi\PostNord\Model\ResourceModel\PersistentQuote\CollectionFactory
     */
    private $persistentQuoteCollectionFactory;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Mediastrategi\PostNord\Api\PersistentQuoteRepositoryInterface $persistentQuoteRepository,
        \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterfaceFactory $persistentQuoteFactory,
        \Mediastrategi\PostNord\Model\ResourceModel\PersistentQuote\CollectionFactory $persistentQuoteCollectionFactory,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepoInterface
    ) {
        parent::__construct($context);
        $this->persistentQuoteRepository =
            $persistentQuoteRepository;
        $this->persistentQuoteFactory =
            $persistentQuoteFactory;
        $this->persistentQuoteCollectionFactory =
            $persistentQuoteCollectionFactory;
        $this->cartRepoInterface =
            $cartRepoInterface;
    }

    public function create(
        int $quoteId,
        array $quoteData
    ): bool {
        $newPersistentQuote =
            $this->persistentQuoteFactory->create();
        $newPersistentQuote->setQuoteId($quoteId);
        $newPersistentQuote->setQuoteData(json_encode($quoteData));
        try {
            $this->persistentQuoteRepository->save($newPersistentQuote);
            return true;
        } catch (\Magento\Framework\Exception\CouldNotSaveException $e) {
            $this->_logger->error(
                __(
                    'Could not save PersistentQuote: %1, Quote-Id: %2, Data: %3',
                    $e->getMessage(),
                    var_export($quoteId, true),
                    var_export($quoteData, true)
                )
            );
            return false;
        }
    }

    public function read(
        int $quoteId
    ): ?\Mediastrategi\PostNord\Api\Data\PersistentQuoteInterface {
        $persistentQuoteCollection =
            $this->persistentQuoteCollectionFactory->create();
        $persistentQuote = $persistentQuoteCollection
            ->addFieldToFilter('quote_id', $quoteId)
            ->getFirstItem();
        if ($persistentQuote->getId()) {
            return $persistentQuote->getDataModel();
        }
        return null;
    }

    public function readData(
        int $quoteId
    ): ?array {
        if ($object = $this->read($quoteId)) {
            try {
                $data = json_decode(
                    $object->getQuoteData(),
                    true
                );
                if (!empty($data)) {
                    return $data;
                }
            } catch (\Exception $e) {
                return null;
            }
        }
        return null;
    }

    public function update(
        int $quoteId,
        array $quoteData
    ): bool {
        if ($object = $this->read($quoteId)) {
            $object->setQuoteData(json_encode($quoteData));
            $this->persistentQuoteRepository->save($object);
            return true;
        } else {
            if (
                $this->create(
                    $quoteId,
                    $quoteData
                )
            ) {
                return true;
            }
        }
        return false;
    }

    public function delete(int $quoteId): bool
    {
        if ($object = $this->read($quoteId)) {
            $this->persistentQuoteRepository->delete($object);
            return true;
        }
        return false;
    }
}
