<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;
use Mediastrategi\PostNord\Api\Data\OrderDataInterfaceFactory;
use Mediastrategi\PostNord\Api\Data\OrderDataSearchResultsInterfaceFactory;
use Mediastrategi\PostNord\Api\OrderDataRepositoryInterface;
use Mediastrategi\PostNord\Model\ResourceModel\OrderData as ResourceOrderData;
use Mediastrategi\PostNord\Model\ResourceModel\OrderData\CollectionFactory as OrderDataCollectionFactory;

class OrderDataRepository implements \Mediastrategi\PostNord\Api\OrderDataRepositoryInterface
{
    protected $resource;
    protected $extensibleDataObjectConverter;
    protected $searchResultsFactory;
    protected $orderDataCollectionFactory;
    private $storeManager;
    protected $dataObjectHelper;
    protected $dataOrderDataFactory;
    protected $dataObjectProcessor;
    protected $orderDataFactory;
    protected $extensionAttributesJoinProcessor;
    private $collectionProcessor;

    public function __construct(
        ResourceOrderData $resource,
        OrderDataFactory $orderDataFactory,
        OrderDataInterfaceFactory $dataOrderDataFactory,
        OrderDataCollectionFactory $orderDataCollectionFactory,
        OrderDataSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource =
            $resource;
        $this->orderDataFactory =
            $orderDataFactory;
        $this->orderDataCollectionFactory =
            $orderDataCollectionFactory;
        $this->searchResultsFactory =
            $searchResultsFactory;
        $this->dataObjectHelper =
            $dataObjectHelper;
        $this->dataOrderDataFactory =
            $dataOrderDataFactory;
        $this->dataObjectProcessor =
            $dataObjectProcessor;
        $this->storeManager =
            $storeManager;
        $this->collectionProcessor =
            $collectionProcessor;
        $this->extensionAttributesJoinProcessor =
            $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter =
            $extensibleDataObjectConverter;
    }

    public function save(
        \Mediastrategi\PostNord\Api\Data\OrderDataInterface $orderData
    ): \Mediastrategi\PostNord\Api\Data\OrderDataInterface {
        $orderDataData = $this->extensibleDataObjectConverter->toNestedArray(
            $orderData,
            [],
            \Mediastrategi\PostNord\Api\Data\OrderDataInterface::class
        );

        $orderDataModel =
            $this->orderDataFactory->create()->setData($orderDataData);

        try {
            $this->resource->save(
                $orderDataModel
            );
        } catch (\Exception $exception) {
            throw new \Magento\Framework\Exception\CouldNotSaveException(
                __(
                    'Could not save the orderData: %1',
                    $exception->getMessage()
                )
            );
        }

        return $orderDataModel->getDataModel();
    }

    public function get(
        $id
    ): \Mediastrategi\PostNord\Api\Data\OrderDataInterface {
        $orderData = $this->orderDataFactory->create();
        $this->resource->load(
            $orderData,
            $id
        );
        if (!$orderData->getId()) {
            throw new \Magento\Framework\Exception\NoSuchEntityException(
                __(
                    'OrderData with id "%1" does not exist.',
                    $id
                )
            );
        }

        return $orderData->getDataModel();
    }

    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ): \Magento\Framework\Api\SearchResults {
        $collection =
            $this->orderDataCollectionFactory->create();

        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \Mediastrategi\PostNord\Api\Data\OrderDataInterface::class
        );

        $this->collectionProcessor->process($criteria, $collection);

        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);

        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }

        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());

        return $searchResults;
    }

    public function delete(
        \Mediastrategi\PostNord\Api\Data\OrderDataInterface $orderData
    ): bool {
        try {
            $orderDataModel =
                $this->orderDataFactory->create();
            $this->resource->load(
                $orderDataModel,
                $orderData->getId()
            );
            $this->resource->delete($orderDataModel);
        } catch (\Exception $exception) {
            throw new \Magento\Framework\Exception\CouldNotDeleteException(
                __(
                    'Could not delete the OrderData: %1',
                    $exception->getMessage()
                )
            );
        }

        return true;
    }

    public function deleteById($id): bool
    {
        return $this->delete($this->get($id));
    }
}
