<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;
use Mediastrategi\PostNord\Api\Data\PersistentQuoteInterfaceFactory;
use Mediastrategi\PostNord\Api\Data\PersistentQuoteSearchResultsInterfaceFactory;
use Mediastrategi\PostNord\Api\PersistentQuoteRepositoryInterface;
use Mediastrategi\PostNord\Model\ResourceModel\PersistentQuote as ResourcePersistentQuote;
use Mediastrategi\PostNord\Model\ResourceModel\PersistentQuote\CollectionFactory as PersistentQuoteCollectionFactory;

class PersistentQuoteRepository implements \Mediastrategi\PostNord\Api\PersistentQuoteRepositoryInterface
{
    protected $resource;
    protected $extensibleDataObjectConverter;
    protected $searchResultsFactory;
    protected $persistentQuoteCollectionFactory;
    private $storeManager;
    protected $dataObjectHelper;
    protected $dataPersistentQuoteFactory;
    protected $dataObjectProcessor;
    protected $persistentQuoteFactory;
    protected $extensionAttributesJoinProcessor;
    private $collectionProcessor;

    public function __construct(
        ResourcePersistentQuote $resource,
        PersistentQuoteFactory $persistentQuoteFactory,
        PersistentQuoteInterfaceFactory $dataPersistentQuoteFactory,
        PersistentQuoteCollectionFactory $persistentQuoteCollectionFactory,
        PersistentQuoteSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource =
            $resource;
        $this->persistentQuoteFactory =
            $persistentQuoteFactory;
        $this->persistentQuoteCollectionFactory =
            $persistentQuoteCollectionFactory;
        $this->searchResultsFactory =
            $searchResultsFactory;
        $this->dataObjectHelper =
            $dataObjectHelper;
        $this->dataPersistentQuoteFactory =
            $dataPersistentQuoteFactory;
        $this->dataObjectProcessor =
            $dataObjectProcessor;
        $this->storeManager =
            $storeManager;
        $this->collectionProcessor =
            $collectionProcessor;
        $this->extensionAttributesJoinProcessor =
            $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter =
            $extensibleDataObjectConverter;
    }

    public function save(
        \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterface $persistentQuote
    ): \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterface {
        $persistentQuoteData = $this->extensibleDataObjectConverter->toNestedArray(
            $persistentQuote,
            [],
            \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterface::class
        );

        $persistentQuoteModel = $this->persistentQuoteFactory->create()->setData($persistentQuoteData);

        try {
            $this->resource->save(
                $persistentQuoteModel
            );
        } catch (\Exception $exception) {
            throw new \Magento\Framework\Exception\CouldNotSaveException(
                __(
                    'Could not save the persistentQuote: %1',
                    $exception->getMessage()
                )
            );
        }

        return $persistentQuoteModel->getDataModel();
    }

    public function get(
        $persistentQuoteId
    ): \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterface {
        $persistentQuote = $this->persistentQuoteFactory->create();
        $this->resource->load(
            $persistentQuote,
            $persistentQuoteId
        );
        if (!$persistentQuote->getId()) {
            throw new \Magento\Framework\Exception\NoSuchEntityException(
                __(
                    'PersistentQuote with id "%1" does not exist.',
                    $persistentQuoteId
                )
            );
        }

        return $persistentQuote->getDataModel();
    }

    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ): \Magento\Framework\Api\SearchResults {
        $collection = $this->persistentQuoteCollectionFactory->create();

        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterface::class
        );

        $this->collectionProcessor->process($criteria, $collection);

        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);

        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }

        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());

        return $searchResults;
    }

    public function delete(
        \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterface $persistentQuote
    ): bool {
        try {
            $persistentQuoteModel = $this->persistentQuoteFactory->create();
            $this->resource->load(
                $persistentQuoteModel,
                $persistentQuote->getId()
            );
            $this->resource->delete($persistentQuoteModel);
        } catch (\Exception $exception) {
            throw new \Magento\Framework\Exception\CouldNotDeleteException(
                __(
                    'Could not delete the PersistentQuote: %1',
                    $exception->getMessage()
                )
            );
        }

        return true;
    }

    public function deleteById($persistentQuoteId): bool
    {
        return $this->delete($this->get($persistentQuoteId));
    }
}
