<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Model;

use Magento\Framework\Api\DataObjectHelper;
use Mediastrategi\PostNord\Api\Data\ScheduleInterface;
use Mediastrategi\PostNord\Api\Data\ScheduleInterfaceFactory;

/**
 * @SuppressWarnings(PHPMD.CamelCasePropertyName)
 * phpcs:disable PSR2.Classes.PropertyDeclaration.Underscore
 */
class Schedule extends \Magento\Framework\Model\AbstractModel
{
    protected $_eventPrefix = 'mediastrategi_postnord_schedule';
    protected $dataObjectHelper;

    protected $scheduleDataFactory;


    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param ScheduleInterfaceFactory $scheduleDataFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param \Mediastrategi\PostNord\Model\ResourceModel\Schedule $resource
     * @param \Mediastrategi\PostNord\Model\ResourceModel\Schedule\Collection $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        ScheduleInterfaceFactory $scheduleDataFactory,
        DataObjectHelper $dataObjectHelper,
        \Mediastrategi\PostNord\Model\ResourceModel\Schedule $resource,
        \Mediastrategi\PostNord\Model\ResourceModel\Schedule\Collection $resourceCollection,
        array $data = []
    ) {
        $this->scheduleDataFactory = $scheduleDataFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Retrieve schedule model with schedule data
     * @return ScheduleInterface
     */
    public function getDataModel()
    {
        $scheduleData = $this->getData();

        $scheduleDataObject = $this->scheduleDataFactory->create();
        $this->dataObjectHelper->populateWithArray(
            $scheduleDataObject,
            $scheduleData,
            ScheduleInterface::class
        );

        return $scheduleDataObject;
    }
}
