<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Model\System\Message;

class CustomMessageNotification implements \Magento\Framework\Notification\MessageInterface
{
    public const MESSAGE_IDENTITY =
        'mspostnord_custom_message_notification';

    /**
     * @var \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    private $urlBuilder;

    /**
     * @var bool
     */
    private $isValidConfiguration = false;

    /**
     * @var bool
     */
    private $isUpdateAvailable = false;

    /**
     * @var bool
     */
    private $isImproperUsage = true;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    private $requestInterface;

    public function __construct(
        \Magento\Framework\UrlInterface $urlBuilder,
        \Mediastrategi\PostNord\Helper\Meta $metaHelper,
        \Magento\Framework\App\RequestInterface $requestInterface
    ) {
        $this->urlBuilder = $urlBuilder;
        $this->metaHelper = $metaHelper;
        $this->requestInterface = $requestInterface;
    }

    public function getIdentity(): string
    {
        return self::MESSAGE_IDENTITY;
    }

    public function isDisplayed(): bool
    {
        $configurationErrors =
            (bool) $this->metaHelper->getConfigurationErrors();
        $referer = $this->requestInterface->getServer(
            'HTTP_REFERER',
            ''
        );
        $refererIsWizard = $referer
            && stripos(
                $referer,
                '/mspostnord/dashboard/'
            )
            !== false;
        $this->isValidConfiguration =
            !$configurationErrors;
        $this->isUpdateAvailable =
            $this->metaHelper->isNewVersionAvailable();
        $this->isImproperUsage =
            !$this->metaHelper->hasProperUsage();
        return (!$refererIsWizard
            && !$this->isValidConfiguration)
            || $this->isUpdateAvailable
            || $this->isImproperUsage;
    }

    public function getText(): string
    {
        $text = '';

        if (!$this->isValidConfiguration) {
            $text .= '<p>'
                . __(
                    'Please setup PostNord Ship Connect in <a href="%1">stores configuration</a>.',
                    $this->urlBuilder->getUrl('adminhtml/system_config/edit/section/carriers/mediastrategi_postnord')
                )
                . '</p>';
        }

        if ($this->isUpdateAvailable) {
            $text .= '<p>'
                . __(
                    'A new version of PostNord Ship Connect is available: %1.',
                    $this->metaHelper->getLatestVersion()
                )
                . '</p>';
        }

        if ($this->isImproperUsage) {
            /* phpcs:disable Generic.Files.LineLength.TooLong */
            $text .= '<p>'
                . __(
                    'Something seems to wrong with your license for PostNord Ship Connect, please verify your license <a href="%1">here</a>.',
                    $this->urlBuilder->getUrl('adminhtml/system_config/edit/section/carriers/mediastrategi_postnord')
                )
                . '</p>';
            /* phpcs:enable Generic.Files.LineLength.TooLong */
        }

        return $text;
    }

    public function getSeverity(): int
    {
        return self::SEVERITY_NOTICE;
    }
}
