<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Plugin\Order\Adminhtml;

class View
{
    /**
     * @var \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    /**
     * @var \Mediastrategi\PostNord\Helper\Shipment
     */
    private $shipmentHelper;

    /**
     * \Mediastrategi\PostNord\Helper\OrderData
     */
    private $orderDataHelper;

    public function __construct(
        \Mediastrategi\PostNord\Helper\Meta $metaHelper,
        \Mediastrategi\PostNord\Helper\OrderData $orderDataHelper,
        \Mediastrategi\PostNord\Helper\Shipment $shipmentHelper
    ) {
        $this->metaHelper = $metaHelper;
        $this->orderDataHelper = $orderDataHelper;
        $this->shipmentHelper = $shipmentHelper;
    }

    /**
     * phpcs:disable Generic.Files.LineLength.TooLong
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function afterToHtml(
        \Magento\Sales\Block\Adminhtml\Order\View\Info $subject,
        string $result
    ): string {
        if ($order = $subject->getOrder()) {
            $shippingMethod = $order->getShippingMethod();
            if (
                empty($shippingMethod)
                || !is_string($shippingMethod)
            ) {
                return $result;
            }

            if (
                $explode = explode(
                    '_',
                    $shippingMethod
                )
            ) {
                $result .= sprintf(
                    '<section id="mspostnord-information-admintml-order-view" class="admin__page-section order-addresses %s" data-ajax-url="%s" data-order-id="%s">'
                        . '<div class="admin__page-section-title"><span class="title">%s</span></div><div class="admin__page-section-content">',
                    $this->metaHelper->isDebug() ? 'is-debug' : 'is-not-debug',
                    $subject->getUrl('mspostnord/misc/options'),
                    $order->getId(),
                    __('PostNord Ship Connect')
                );

                $result .= sprintf(
                    '<script type="text/json" id="mspostnord-order-i18n">%s</script>',
                    json_encode(
                        [
                            'delivery_text' =>
                                __('Delivery:'),
                            'Found no shipping-options for order!' =>
                                __('Found no shipping-options for order!'),
                            'is_recommended' =>
                                __('Recommended'),
                            'Missing required data to save!' =>
                                __('Missing required data to save!'),
                            'Please select a shipment option first!' =>
                                __('Please select a shipment option first!'),
                            'Select Shipping-option' =>
                                __('Select Shipping-option'),
                        ]
                    )
                );

                // Change shipping method
                $result .= sprintf(
                    '<div class="admin__page-section-item order-information mspostnord-change-shipping-method"><div class="admin__page-section-item-title">'
                        . '<span class="title">%s</span></div><div class="admin__page-section-item-content"><div class="widget-container"><p class="no-options-loaded">%s</p></div>'
                        . '<button class="action-default scalable action-search-method action-secondary">%s</button>&nbsp;'
                        . '<button disabled="disabled" class="action-default scalable action-search-save action-secondary">%s</button></div></div>',
                    __('Change Shipping Method'),
                    __('No options loaded'),
                    __('Load available options'),
                    __('Save new method')
                );

                // Shipping method information
                if (
                    !empty($explode[0])
                    && !empty($explode[1])
                    && $explode[0] == 'mspostnord'
                ) {
                    if ($explode[1] === 'fallback') {
                        $result .= sprintf(
                            '<div class="admin__page-section-item mspostnord-order-shipping-method view-information"><div class="admin__page-section-item-title">'
                                . '<span class="title">%s</span></div><div class="admin__page-section-item-content"><p>%s</p></div></div>',
                            __('Information'),
                            __('This order uses a fallback method that is available only when there are communication-problems with PostNord. Please change shipping option on this order.')
                        );
                    } else {
                        $orderData = $this->orderDataHelper->readData(
                            (int) $order->getEntityId()
                        );

                        // Update shipments statuses if shipments is available on the order
                        if (
                            !empty($orderData['shipments'])
                            && is_array($orderData['shipments'])
                        ) {
                            $ordersAndShipments = [];
                            foreach ($orderData['shipments'] as $shipment) {
                                if (
                                    is_array($shipment)
                                    && !empty($shipment['id'])
                                    && is_string($shipment['id'])
                                ) {
                                    $ordersAndShipments[] = [
                                        (int) $order->getEntityId(),
                                        $shipment['id']
                                    ];
                                }
                            }
                            if ($ordersAndShipments) {
                                $this->shipmentHelper->flag = 0;
                                $this->shipmentHelper->shipmentStatuses(
                                    $ordersAndShipments,
                                    true
                                );

                                // Do we need to refresh? (to update order buttons)
                                if ($this->shipmentHelper->flag === 1) {
                                    $result = '<script type="text/javascript">location.reload(true);</script>';
                                }
                            }
                        }

                        $result .= sprintf(
                            '<div class="admin__page-section-item order-shipping-method view-information"><div class="admin__page-section-item-title">'
                                . '<span class="title">%s</span></div><div class="admin__page-section-item-content"><dl>',
                            __('Information')
                        );

                        $result .= '<dt class="option-id">' . __('Shipment Option ID') . '</dt><dd class="option-id ' . $explode[1] . '">' . $explode[1] . '</dd>';

                        if (
                            !empty($orderData['servicePointId'])
                            && is_string($orderData['servicePointId'])
                        ) {
                            $result .= '<dt class="service-point-id">' . __('Service Point ID') . '</dt><dd class="service-point-id">' . $orderData['servicePointId'] . '</dd>';
                        }

                        if (
                            !empty($orderData['servicePointName'])
                            && is_string($orderData['servicePointName'])
                        ) {
                            $result .= '<dt class="service-point-name">' . __('Service Point Name') . '</dt><dd class="service-point-name">' . $orderData['servicePointName'] . '</dd>';
                        }

                        if (
                            !empty($orderData['addons'])
                            && is_array($orderData['addons'])
                        ) {
                            $addons = $orderData['addons'];
                            $result .= '<dt class="addons">' . __('Add-ons') . '</dt><dd class="addons"><ul>';
                            foreach ($addons as $addonId => $addon) {
                                $result .= sprintf(
                                    '<li><strong class="addon-name">%s (%s)</strong>',
                                    $addon['name'] ?? '-',
                                    $addonId
                                );
                                if (!empty($addon['helpText'])) {
                                    $result .= sprintf(
                                        '<br /><em class="addon-helptext">%s</em>',
                                        $addon['helpText']
                                    );
                                }
                                if (
                                    isset($addon['value'])
                                    && $addon['value'] !== true
                                ) {
                                    $result .= sprintf(
                                        '<br /><span class="addon-value">%s: %s</span>',
                                        __('Value'),
                                        $addon['value']
                                    );
                                }
                            }
                            $result .= '</ul></dd>';
                        }

                        $result .= '</dl>';
                        $result .= '</div></div>';
                    }
                }
            }

            $result .= '</div></section>';
        }

        return $result;
    }
}
