<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Plugin\Order;

class Get
{
    /**
     * @var \Mediastrategi\PostNord\Model\Order\MspostnordAssignmentFactory
     */
    private $mspostnordAssignmentFactory;

    /**
     * \Mediastrategi\PostNord\Helper\OrderData
     */
    private $orderDataHelper;

    /**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */
    private $orderExtensionFactory;

    public function __construct(
        \Mediastrategi\PostNord\Helper\OrderData $orderDataHelper,
        \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
        \Mediastrategi\PostNord\Model\Order\MspostnordAssignmentFactory $mspostnordAssignmentFactory
    ) {
        $this->orderDataHelper = $orderDataHelper;
        $this->orderExtensionFactory = $orderExtensionFactory;
        $this->mspostnordAssignmentFactory = $mspostnordAssignmentFactory;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFunctionParameter)
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter
     */
    public function afterGet(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderInterface $resultOrder
    ): \Magento\Sales\Api\Data\OrderInterface {
        return $this->getMspostnordAssignments($resultOrder);
    }

    public function afterGetList(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Model\ResourceModel\Order\Collection $resultOrder
    ): \Magento\Sales\Model\ResourceModel\Order\Collection {
        foreach ($resultOrder->getItems() as $order) {
            $this->afterGet(
                $subject,
                $order
            );
        }

        return $resultOrder;
    }

    private function getMspostnordAssignments(
        \Magento\Sales\Api\Data\OrderInterface $order
    ): \Magento\Sales\Api\Data\OrderInterface {
        $extensionAttributes =
            $order->getExtensionAttributes();
        $orderExtension =
            $extensionAttributes ?: $this->orderExtensionFactory->create();
        $mspostnordAssignments =
            $this->mspostnordAssignmentFactory->create();
        $orderId =
            (int) $order->getEntityId();
        $orderData =
            $this->orderDataHelper->readData($orderId);

        if (
            !empty($orderData['addons'])
            && is_array($orderData['addons'])
        ) {
            $mspostnordAssignments->setAdditionalServices($orderData['addons']);
        }

        if (
            !empty($orderData['servicePointId'])
            && is_string($orderData['servicePointId'])
        ) {
            $mspostnordAssignments->setServicePointId($orderData['servicePointId']);
        }

        if (
            !empty($orderData['servicePointName'])
            && is_string($orderData['servicePointName'])
        ) {
            $mspostnordAssignments->setServicePointName($orderData['servicePointName']);
        }

        $orderExtension->setMspostnordAssignments($mspostnordAssignments);
        $order->setExtensionAttributes($orderExtension);
        return $order;
    }
}
