<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Plugin\Order;

class Save
{
    /**
     * @var \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    /**
     * \Mediastrategi\PostNord\Helper\OrderData
     */
    private $orderDataHelper;

    /**
     * @var \Mediastrategi\PostNord\Helper\PersistentQuote
     */
    private $persistentQuoteHelper;

    public function __construct(
        \Mediastrategi\PostNord\Helper\Meta $metaHelper,
        \Mediastrategi\PostNord\Helper\OrderData $orderDataHelper,
        \Mediastrategi\PostNord\Helper\PersistentQuote $persistentQuoteHelper
    ) {
        $this->metaHelper = $metaHelper;
        $this->orderDataHelper = $orderDataHelper;
        $this->persistentQuoteHelper = $persistentQuoteHelper;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter
     */
    public function afterSave(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        \Magento\Sales\Api\Data\OrderInterface $order
    ): \Magento\Sales\Api\Data\OrderInterface {
        if ($this->metaHelper->isEnabledForCheckout()) {
            $quoteId = (int) $order->getQuoteId();
            $shippingMethod = $order->getShippingMethod();
            if (
                empty($shippingMethod)
                || !is_string($shippingMethod)
            ) {
                return $order;
            }
            $methodExplode = explode(
                '_',
                $shippingMethod
            );
            $shippingCarrierId =
                $methodExplode[0];
            $shippingMethodId =
                $methodExplode[1];
            $this->metaHelper->log(
                (string) __(
                    '%1, quote-id: %2, shipping-method: %3',
                    __METHOD__,
                    $quoteId,
                    $shippingMethod
                )
            );
            if ($shippingCarrierId === 'mspostnord') {
                $this->metaHelper->log(
                    (string) __(
                        '%1 PostNord Ship Connect is used for quote',
                        __METHOD__
                    )
                );
                if ($shippingMethodId === 'fallback') {
                    $this->metaHelper->log(
                        (string) __(
                            '%1 shipping-option is fallback, ignore checkout state',
                            __METHOD__
                        )
                    );
                } else {
                    $this->metaHelper->log(
                        (string) __(
                            '%1 shipping-option id is "%2"',
                            __METHOD__,
                            $shippingMethodId
                        )
                    );

                    if ($state = $this->persistentQuoteHelper->readData($quoteId)) {
                        $this->metaHelper->log(
                            (string) __(
                                '%1 did find checkout state data: "%2"',
                                __METHOD__,
                                var_export($state, true)
                            )
                        );
                        $orderData = [];
                        $optionId = $shippingMethodId;
                        if (isset($state[$optionId])) {
                            $this->metaHelper->log(
                                (string) __(
                                    '%1 did find state data for selected shipping-option',
                                    __METHOD__
                                )
                            );
                            $optionState =
                                $state[$optionId];

                            // Selected add-ons data
                            if (isset($optionState['selectedAddons'])) {
                                $this->metaHelper->log(
                                    (string) __(
                                        '%1 did find state data for selected add-ons',
                                        __METHOD__
                                    )
                                );
                                $selectedAddons = [];
                                foreach ($optionState['selectedAddons'] as $selectedAddonId => $selectedAddon) {
                                    if (!empty($selectedAddon['value'])) {
                                        // Convert string booleans to real booleans
                                        if ($selectedAddon['value'] === 'true') {
                                            $selectedAddon['value'] = true;
                                        } elseif ($selectedAddon['value'] === 'false') {
                                            $selectedAddon['value'] = false;
                                        }

                                        $selectedAddons[$selectedAddonId] =
                                            $selectedAddon;
                                    }
                                }

                                if (!empty($selectedAddons)) {
                                    $this->metaHelper->log(
                                        (string) __(
                                            '%1 did find selected add-ons %2',
                                            __METHOD__,
                                            var_export($selectedAddons, true)
                                        )
                                    );
                                    $orderData['addons'] =
                                        $selectedAddons;
                                } else {
                                    $this->metaHelper->log(
                                        (string) __(
                                            '%1 did not find selected add-ons',
                                            __METHOD__
                                        )
                                    );
                                }
                            } else {
                                $this->metaHelper->log(
                                    (string) __(
                                        '%1 did not find state data for selected add-ons',
                                        __METHOD__
                                    )
                                );
                            }

                            // Select service-point data
                            if (isset($optionState['selectedServicePoint'])) {
                                $this->metaHelper->log(
                                    (string) __(
                                        '%1 did find state data for selected service-point',
                                        __METHOD__
                                    )
                                );

                                // Service-point id
                                if (isset($optionState['selectedServicePoint']['servicePointId'])) {
                                    $this->metaHelper->log(
                                        (string) __(
                                            '%1 did find id for selected service-point',
                                            __METHOD__
                                        )
                                    );
                                    $orderData['servicePointId'] =
                                        (string) $optionState['selectedServicePoint']['servicePointId'];
                                } else {
                                    $this->metaHelper->log(
                                        (string) __(
                                            '%1 did not find id for selected service-point',
                                            __METHOD__
                                        )
                                    );
                                }

                                // Service-point name
                                if (isset($optionState['selectedServicePoint']['formattedName'])) {
                                    $this->metaHelper->log(
                                        (string) __(
                                            '%1 did find name for selected service-point',
                                            __METHOD__
                                        )
                                    );
                                    $orderData['servicePointName'] =
                                        (string) $optionState['selectedServicePoint']['formattedName'];
                                } else {
                                    $this->metaHelper->log(
                                        (string) __(
                                            '%1 did not find name for selected service-point',
                                            __METHOD__
                                        )
                                    );
                                }
                            } else {
                                $this->metaHelper->log(
                                    (string) __(
                                        '%1 did not find data for selected service-point',
                                        __METHOD__
                                    )
                                );
                            }

                            if ($orderData) {
                                $this->metaHelper->log(
                                    (string) __(
                                        '%1 found order data',
                                        __METHOD__,
                                        var_export($orderData, true)
                                    )
                                );
                                $this->orderDataHelper->update(
                                    (int) $order->getEntityId(),
                                    $orderData
                                );
                            }
                            $this->metaHelper->log(
                                (string) __(
                                    '%1 updated %2 order attributes from saved state',
                                    __METHOD__,
                                    count($orderData)
                                )
                            );
                        } else {
                            $this->metaHelper->log(
                                (string) __(
                                    '%1 did not find state data for selected option id',
                                    __METHOD__
                                )
                            );
                        }

                        if (
                            $this->persistentQuoteHelper->delete(
                                $quoteId
                            )
                        ) {
                            $this->metaHelper->log(
                                (string) __(
                                    '%1 successfully deleted persistent state data for quote id %2',
                                    __METHOD__,
                                    $quoteId
                                )
                            );
                        } else {
                            $this->metaHelper->log(
                                (string) __(
                                    '%1 failed to delete persistent state data for quote id %2',
                                    __METHOD__,
                                    $quoteId
                                )
                            );
                        }
                    } else {
                        $this->metaHelper->log(
                            (string) __(
                                '%1 did not find shipping state data',
                                __METHOD__
                            )
                        );
                    }
                }
            } else {
                $this->metaHelper->log(
                    (string) __(
                        '%1 PostNord Ship Connect is not used for order',
                        __METHOD__
                    )
                );
            }
        }

        return $order;
    }
}
