<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Plugin;

class ShippingMethodConverterPlugin
{
    /**
     * @var \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    /**
     * @var \Magento\Framework\Api\ExtensionAttributesFactory
     */
    private $extensionAttributesFactory;

    public function __construct(
        \Mediastrategi\PostNord\Helper\Meta $metaHelper,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionAttributesFactory
    ) {
        $this->metaHelper =
            $metaHelper;
        $this->extensionAttributesFactory =
            $extensionAttributesFactory;
    }

    /**
     * @see \Mediastrategi\PostNord\Model\Carrier->collectRates()
     * @see \Magento\Quote\Model\Quote\Address->getShippingRateByCode()
     * @see \Magento\Quote\Model\Quote\Address->getShippingRatesCollection()
     * @see \Magento\Quote\Model\Quote->requestShippingRates()
     * @see \Magento\Quote\Model\Quote\Address\Rate->importShippingRate()
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter.FoundBeforeLastUsed
     */
    public function aroundModelToDataObject(
        \Magento\Quote\Model\Cart\ShippingMethodConverter $subject,
        callable $proceed,
        \Magento\Quote\Model\Quote\Address\Rate $rateModel,
        string $quoteCurrencyCode
    ): \Magento\Quote\Api\Data\ShippingMethodInterface {
        $result = $proceed(
            $rateModel,
            $quoteCurrencyCode
        );
        if ($this->metaHelper->isEnabledForCheckout()) {
            $extensionAttributes = $result->getExtensionAttributes();
            if (!isset($extensionAttributes)) {
                $extensionAttributes = $this->extensionAttributesFactory->create(
                    \Magento\Quote\Api\Data\ShippingMethodInterface::class
                );
            }
            $extensionAttributes->setMspostnordShipmentOptionId(
                $rateModel->getMspostnordShipmentOptionId()
            );
            $extensionAttributes->setMspostnordSelectedServicePoint(
                $rateModel->getMspostnordSelectedServicePoint()
            );
            $extensionAttributes->setMspostnordAvailableServicePoints(
                $rateModel->getMspostnordAvailableServicePoints()
            );
            $extensionAttributes->setMspostnordSelectedAddons(
                $rateModel->getMspostnordSelectedAddons()
            );
            $extensionAttributes->setMspostnordAvailableAddons(
                $rateModel->getMspostnordAvailableAddons()
            );
            $extensionAttributes->setMspostnordLongDescription(
                $rateModel->getMspostnordLongDescription()
            );
            $extensionAttributes->setMspostnordIsRecommended(
                $rateModel->getMspostnordIsRecommended()
            );
            $extensionAttributes->setMspostnordDeliveryEstimateText(
                $rateModel->getMspostnordDeliveryEstimateText()
            );
            $result->setExtensionAttributes($extensionAttributes);
        }

        return $result;
    }
}
