<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Plugin;

class ShippingRateCarrierResultPlugin
{
    /**
     * @var \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    public function __construct(
        \Mediastrategi\PostNord\Helper\Meta $metaHelper
    ) {
        $this->metaHelper = $metaHelper;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter.FoundBeforeLastUsed
     * @return \Magento\Quote\Model\Quote\Address\RateResult\Method[]
     */
    public function aroundGetAllRates(
        \Magento\Shipping\Model\Rate\CarrierResult $subject,
        callable $proceed
    ) {
        $result = $proceed();
        if ($this->metaHelper->isEnabledForCheckout()) {
            /**
             * Get last options from the API
             * to be able to revert sorting
             */
            $options =
                $this->metaHelper->getLastShippingOptionsResult();
            $optionsCount =
                count($options);
            $postNordOptionOrder =
                [];

            // Build order value for each option in PostNord
            foreach ($options as $i => $option) {
                $postNordOptionOrder[$option['shipmentOptionId']] =
                    $i;
            }

            /**
             * Build sort value for each rate
             * PostNord options should come before
             * other options
             */
            $magentoOptionOrder = [];
            foreach ($result as $i => $rate) {
                $optionId =
                    $rate->getMspostnordShipmentOptionId();
                if ($optionId) {
                    $magentoOptionOrder[$i] =
                        $postNordOptionOrder[$optionId];
                } else {
                    $magentoOptionOrder[$i] =
                        $optionsCount + 1;
                }
            }

            /**
             * Sort indexes by sort value ascending order
             * and maintain array index association
             */
            asort($magentoOptionOrder);

            // Build new result from new sorting
            $newResult = [];
            foreach (array_keys($magentoOptionOrder) as $rateIndex) {
                $newResult[] =
                    $result[$rateIndex];
            }
            $result = $newResult;
        }
        return $result;
    }
}
