<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Test\Unit;

class PersistentQuoteTest extends \PHPUnit\Framework\TestCase
{
    /**
     * @var \Mediastrategi\PostNord\Helper\PersistentQuote
     */
    private $persistentQuoteHelper;

    protected function setUp(): void
    {
        $objectManager =
            new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);
        $persistentQuote = $this->createPartialMock(
            \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterface::class,
            [
                'getExtensionAttributes',
                'getId',
                'getQuoteData',
                'getQuoteId',
                'setExtensionAttributes',
                'setId',
                'setQuoteData',
                'setQuoteId',
            ]
        );
        $persistentQuote
        ->method('getExtensionAttributes')
            ->will($this->returnValue([]));
        $persistentQuote
        ->method('getId')
            ->will($this->returnValue(1));
        $persistentQuote
        ->method('getQuoteData')
            ->will($this->returnValue(json_encode(["a" => 123, "b" => 456])));
        $persistentQuote
        ->method('getQuoteId')
            ->will($this->returnValue(1));
        $persistentQuote
        ->method('setExtensionAttributes')
            ->will($this->returnValue($persistentQuote));
        $persistentQuote
        ->method('setId')
            ->will($this->returnValue($persistentQuote));
        $persistentQuote
        ->method('setQuoteData')
            ->will($this->returnValue($persistentQuote));
        $persistentQuote
        ->method('setQuoteId')
            ->will($this->returnValue($persistentQuote));

        $persistentQuoteRepository = $this->createPartialMock(
            \Mediastrategi\PostNord\Api\PersistentQuoteRepositoryInterface::class,
            [
                'delete',
                'deleteById',
                'get',
                'getList',
                'save'
            ]
        );
        $persistentQuoteRepository
        ->method('delete')
            ->will($this->returnValue(true));
        $persistentQuoteRepository
        ->method('deleteById')
            ->will($this->returnValue(true));
        $persistentQuoteRepository
        ->method('get')
            ->will($this->returnValue($persistentQuote));
        $persistentQuoteRepository
        ->method('getList')
            ->will($this->returnValue([$persistentQuote]));
        $persistentQuoteRepository
        ->method('save')
            ->will($this->returnValue(true));

        $persistentQuoteFactory = $this->createPartialMock(
            \Mediastrategi\PostNord\Api\Data\PersistentQuoteInterfaceFactory::class,
            ['create']
        );
        $persistentQuoteFactory->expects($this->any())
            ->method('create')
            ->will($this->returnValue($persistentQuote));

        $searchCriteria = $this->createMock(
            \Magento\Framework\Api\SearchCriteriaInterface::class
        );
        $searchCriteriaBuilder = $this->createPartialMock(
            \Magento\Framework\Api\SearchCriteriaBuilder::class,
            ['create']
        );
        $searchCriteriaBuilder->expects($this->any())
            ->method('create')
            ->will($this->returnValue($searchCriteria));

        $persistentQuoteModel = $this->createPartialMock(
            \Mediastrategi\PostNord\Model\PersistentQuote::class,
            [
                'getDataModel',
                'getId',
            ]
        );
        $persistentQuoteModel
        ->method('getDataModel')
            ->will($this->returnValue($persistentQuote));
        $persistentQuoteModel
        ->method('getId')
            ->will($this->returnValue(1));

        $persistentQuoteCollection = $this->createPartialMock(
            \Mediastrategi\PostNord\Model\ResourceModel\PersistentQuote\Collection::class,
            [
                'addFieldToFilter',
                'getFirstItem'
            ]
        );
        $persistentQuoteCollection
        ->method('addFieldToFilter')
            ->will($this->returnValue($persistentQuoteCollection));
        $persistentQuoteCollection
        ->method('getFirstItem')
            ->will($this->returnValue($persistentQuoteModel));

        $persistentQuoteCollectionFactory = $this->createPartialMock(
            \Mediastrategi\PostNord\Model\ResourceModel\PersistentQuote\CollectionFactory::class,
            [
                'create'
            ]
        );
        $persistentQuoteCollectionFactory
        ->method('create')
            ->will($this->returnValue($persistentQuoteCollection));

        $this->persistentQuoteHelper = $objectManager->getObject(
            \Mediastrategi\PostNord\Helper\PersistentQuote::class,
            [
                'persistentQuoteCollectionFactory' =>
                    $persistentQuoteCollectionFactory,
                'persistentQuoteFactory' =>
                    $persistentQuoteFactory,
                'persistentQuoteRepository' =>
                    $persistentQuoteRepository,
                'searchCriteriaBuilder' =>
                    $searchCriteriaBuilder,
            ]
        );
    }

    public function testCreate()
    {
        // Prep
        $quoteData = [
            'attr'  => 'data',
            'attr2' => 'data2',
        ];

        // Action
        $result = $this->persistentQuoteHelper->create(
            1,
            $quoteData
        );

        // Assert
        $this->assertTrue($result);
    }

    public function testRead()
    {
        $result = $this->persistentQuoteHelper->read(1);
        $this->assertNotNull($result);
    }

    public function testReadData()
    {
        $result = $this->persistentQuoteHelper->readData(1);
        $this->assertEquals(
            $result,
            ["a" => 123, "b" => 456]
        );
    }

    public function testUpdate()
    {
        // Prep
        $quoteData = [
            'attr'  => 'data',
            'attr2' => 'data3',
        ];
        $this->assertTrue(
            $this->persistentQuoteHelper->update(
                1,
                $quoteData
            )
        );
    }

    public function testDelete()
    {
        $this->assertTrue(
            $this->persistentQuoteHelper->delete(1)
        );
    }
}
