<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Ui\Component\Listing\Column;

class Data extends \Magento\Ui\Component\Listing\Columns\Column
{
    /**
     * @var \Magento\Backend\Helper\Data
     */
    private $backendHelper;

    /**
     * \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * \Mediastrategi\PostNord\Helper\OrderData
     */
    private $orderDataHelper;

    public function __construct(
        \Magento\Framework\View\Element\UiComponent\ContextInterface $context,
        \Magento\Framework\View\Element\UiComponentFactory $uiComponentFactory,
        \Magento\Backend\Helper\Data $backendHelper,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Mediastrategi\PostNord\Helper\OrderData $orderDataHelper,
        \Mediastrategi\PostNord\Helper\Meta $metaHelper,
        array $components = [],
        array $data = []
    ) {
        parent::__construct(
            $context,
            $uiComponentFactory,
            $components,
            $data
        );
        $this->backendHelper = $backendHelper;
        $this->metaHelper = $metaHelper;
        $this->orderRepository = $orderRepository;
        $this->orderDataHelper = $orderDataHelper;
    }

    public function prepareDataSource(array $dataSource): array
    {
        if (!empty($dataSource['data']['items'])) {
            foreach ($dataSource['data']['items'] as &$item) {
                if (!empty($item['entity_id'])) {
                    $orderId =
                        (int) $item['entity_id'];
                    $orderData = $this->orderDataHelper->readData(
                        $orderId
                    );
                    if ($orderData) {
                        $html = '';
                        if (
                            !empty($orderData['shipments'])
                            && is_array($orderData['shipments'])
                        ) {
                            $shipmentsHtml = '';
                            foreach ($orderData['shipments'] as $shipment) {
                                if (
                                    is_array($shipment)
                                    && !empty($shipment['status'])
                                    && is_string($shipment['status'])
                                    && !empty($shipment['id'])
                                    && is_string($shipment['id'])
                                ) {
                                    $shipmentsHtml .= '<li><dl>';
                                    $shipmentsHtml .= sprintf(
                                        '<dt class="id">%s</dt><dd class="id">%s</dd>',
                                        __('Shipment'),
                                        $shipment['id']
                                    );
                                    $shipmentsHtml .= sprintf(
                                        '<dt class="status">%s</dt><dd class="status">%s</dd>',
                                        __('Status'),
                                        $this->metaHelper->getStatusText($shipment['status'])
                                    );
                                    if (
                                        $shipment['status'] ==
                                        \Mediastrategi\PostNord\Helper\Api::SHIPMENT_STATUS_BOOKED
                                        && !empty($shipment['printedLabel'])
                                    ) {
                                        $shipmentsHtml .= sprintf(
                                            '<dt class="label">%s</dt>'
                                                . '<dd class="label">'
                                                . '<a onclick="(event.stopPropagation() '
                                                . '|| event.cancelBubble);" target="_blank" '
                                                . 'href="%s">%s</a></dd>',
                                            __(
                                                'Label'
                                            ),
                                            $this->backendHelper->getUrl(
                                                'mspostnord/shipment/label',
                                                [
                                                    'id' =>
                                                        $shipment['id'],
                                                    'order' =>
                                                        $orderId,
                                                ]
                                            ),
                                            __(
                                                'Download'
                                            )
                                        );
                                    }

                                    // Tracking links
                                    if (
                                        !empty($shipment['trackingIds'])
                                        && is_array($shipment['trackingIds'])
                                    ) {
                                        foreach ($shipment['trackingIds'] as $trackingId) {
                                            $shipmentsHtml .= sprintf(
                                                '<dt class="tracking">%s</dt>'
                                                    . '<dd class="tracking">'
                                                    . '<a onclick="(event.stopPropagation() '
                                                    . '|| event.cancelBubble);" target="_blank" '
                                                    . 'href="%s">%s</a></dd>',
                                                __('Tracking'),
                                                sprintf(
                                                    \Mediastrategi\PostNord\Helper\Shipment::TRACKING_LINK_FORMAT,
                                                    $trackingId,
                                                ),
                                                __('Link')
                                            );
                                        }
                                    }

                                    $shipmentsHtml .= '</dl></li>';
                                }
                            }

                            if ($shipmentsHtml) {
                                $html .= '<ul class="mspostnord-shipments">'
                                    . $shipmentsHtml
                                    . '</ul>';
                            }

                            if (!empty($orderData['ignoreAutomation'])) {
                                $html .= '<p class="ignored">' . __('Ignored in automation') . '</p>';
                            }

                            $item['mspostnord'] = $html;
                        }
                    }
                }
            }
        }
        return $dataSource;
    }
}
