<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\ViewModel\Adminhtml;

class Automation implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    /**
     * \Mediastrategi\PostNord\Helper\OrderData
     */
    private $orderDataHelper;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     */
    private $orderRepository;

    /**
     * @var \Mediastrategi\PostNord\Api\OrderDataRepositoryInterface
     */
    private $orderDataRepositoryInterface;

    /**
     * @var \Mediastrategi\PostNord\Api\PersistentQuoteRepositoryInterface
     */
    private $persistentQuoteRepositoryInterface;

    /**
     * @var \Magento\Quote\Api\CartRepositoryInterface
     */
    private $cartRepositoryInterface;

    /**
     * @var \Mediastrategi\PostNord\Helper\PersistentQuote
     */
    private $persistentQuoteHelper;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\Api\SortOrderBuilder
     */
    private $sortOrderBuilder;

    /**
     * @var \Magento\Shipping\Model\Shipping\LabelGenerator
     */
    private $labelGenerator;

    /**
     * @var \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    /**
     * @var \Mediastrategi\PostNord\Helper\Shipment
     */
    private $shipmentHelper;

    /**
     * @var \Magento\Sales\Model\Convert\Order
     */
    private $orderConverter;

    /**
     * @var \Magento\Shipping\Model\ShipmentNotifier
     */
    private $shipmentNotifier;

    /**
     * @var \Magento\Framework\Stdlib\Cookie\CookieReaderInterface
     */
    private $cookieReaderInterface;

    /**
     * @var \Magento\Framework\Stdlib\StringUtils
     */
    private $stringUtils;

    /**
     * @var \Magento\Framework\App\Route\ConfigInterface\Proxy
     */
    private $configInterfaceProxy;

    /**
     * @var \Magento\Framework\App\Request\PathInfoProcessorInterface
     */
    private $pathInfoProcessorInterface;

    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    private $objectManagerInterface;

    /**
     * @var \Magento\User\Model\User
     */
    private $userModel;

    /**
     * @var \Magento\Backend\Model\Auth\Session
     */
    private $authSession;

    /**
     * @var \Magento\Store\Model\StoreManager
     */
    private $storeManager;

    /**
     * @var string
     */
    private $log = '';

    /**
     * @var \Magento\Catalog\Model\ProductRepository
     */
    private $productRepository;

    /**
     * @var \Mediastrategi\PostNord\Helper\Schedule
     */
    private $scheduleHelper;
    /**
     * @var \Mediastrategi\PostNord\Helper\ApiTransaction
     */
    private $apiTransactionHelper;

    public function __construct(
        \Mediastrategi\PostNord\Helper\OrderData $orderDataHelper,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\SortOrderBuilder $sortOrderBuilder,
        \Magento\Shipping\Model\Shipping\LabelGenerator $labelGenerator,
        \Magento\Sales\Model\Convert\Order $orderConverter,
        \Magento\Shipping\Model\ShipmentNotifier $shipmentNotifier,
        \Magento\Framework\Stdlib\Cookie\CookieReaderInterface $cookieReaderInterface,
        \Magento\Framework\Stdlib\StringUtils $stringUtils,
        \Magento\Framework\App\Request\PathInfoProcessorInterface $pathInfoProcessorInterface,
        \Magento\Framework\ObjectManagerInterface $objectManagerInterface,
        \Magento\User\Model\User $userModel,
        \Magento\Backend\Model\Auth\Session $authSession,
        \Magento\Store\Model\StoreManager $storeManager,
        \Mediastrategi\PostNord\Helper\Meta $metaHelper,
        \Magento\Catalog\Model\ProductRepository $productRepository,
        \Mediastrategi\PostNord\Helper\Schedule $scheduleHelper,
        \Mediastrategi\PostNord\Helper\ApiTransaction $apiTransactionHelper,
        \Mediastrategi\PostNord\Helper\Shipment $shipmentHelper,
        \Mediastrategi\PostNord\Api\OrderDataRepositoryInterface $orderDataRepositoryInterface,
        \Mediastrategi\PostNord\Api\PersistentQuoteRepositoryInterface $persistentQuoteRepositoryInterface,
        \Mediastrategi\PostNord\Helper\PersistentQuote $persistentQuoteHelper,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepositoryInterface
    ) {
        $this->orderDataHelper = $orderDataHelper;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->sortOrderBuilder = $sortOrderBuilder;
        $this->labelGenerator = $labelGenerator;
        $this->orderConverter = $orderConverter;
        $this->shipmentNotifier = $shipmentNotifier;
        $this->cookieReaderInterface = $cookieReaderInterface;
        $this->stringUtils = $stringUtils;
        $this->configInterfaceProxy = new \Magento\Framework\App\Route\ConfigInterface\Proxy(
            $objectManagerInterface,
            \Magento\Framework\App\Route\ConfigInterface::class,
            true
        );
        $this->pathInfoProcessorInterface = $pathInfoProcessorInterface;
        $this->objectManagerInterface = $objectManagerInterface;
        $this->userModel = $userModel;
        $this->authSession = $authSession;
        $this->storeManager = $storeManager;
        $this->metaHelper = $metaHelper;
        $this->productRepository = $productRepository;
        $this->scheduleHelper = $scheduleHelper;
        $this->apiTransactionHelper = $apiTransactionHelper;
        $this->shipmentHelper = $shipmentHelper;
        $this->orderDataRepositoryInterface =
            $orderDataRepositoryInterface;
        $this->persistentQuoteRepositoryInterface =
            $persistentQuoteRepositoryInterface;
        $this->persistentQuoteHelper =
            $persistentQuoteHelper;
        $this->cartRepositoryInterface =
            $cartRepositoryInterface;
    }

    public function getLog(): string
    {
        return $this->log;
    }

    public function execute()
    {
        $this->log = '';
        $this->log((string) __('Automation started'));

        $successCount = 0;
        $errorCount   = 0;

        foreach ($this->storeManager->getStores() as $storeItem) {
            $storeId = $storeItem->getId();
            $this->log(
                (string) __(
                    'Switching to store %1 (%2)',
                    $storeItem->getName(),
                    $storeId
                )
            );
            $this->storeManager->setCurrentStore($storeId);

            // Iterate through shipping methods here
            if ($this->metaHelper->getConfigurationErrors()) {
                $this->log(
                    (string) __('Configuration is invalid in store, skipping store')
                );
                $errorCount++;
                continue;
            }
            if (
                $this->metaHelper->getConfig(
                    'carriers/mspostnord/automation/enabled'
                )
            ) {
                $status = $this->metaHelper->getConfig(
                    'carriers/mspostnord/automation/order_status'
                );
                $this->log((string) __('Automation enabled in this store'));
                $this->log((string) __('Fetching orders..'));

                $criteria = $this->searchCriteriaBuilder
                    ->addFilter('status', $status)
                        ->addFilter('shipping_method', 'mspostnord_%', 'like')
                        ->addFilter('store_id', $storeId)
                        ->create();

                if ($orders = $this->orderRepository->getList($criteria)->getItems()) {
                    foreach ($orders as $order) {
                        $shippingMethod = $order->getShippingMethod();
                        if (
                            empty($shippingMethod)
                            || !is_string($shippingMethod)
                        ) {
                            $this->log(
                                (string) __(
                                    'Skipping order %1 because it is missing a shipping method',
                                    (int) $order->getEntityId()
                                )
                            );
                            continue;
                        }

                        // Only proceed if order is not a fallback option
                        $explode = explode(
                            '_',
                            $shippingMethod
                        );
                        if (
                            !empty($explode[0])
                            && !empty($explode[1])
                            && $explode[0] == 'mspostnord'
                            && $explode[1] !== 'fallback'
                        ) {
                            $this->log(
                                (string) __(
                                    'Order %1 is not using a fallback shipping-option',
                                    (int) $order->getEntityId()
                                )
                            );
                            $orderData = $this->orderDataHelper->readData(
                                (int) $order->getEntityId()
                            );
                            $ignoreAutomation =
                                !empty($orderData['ignoreAutomation']);
                            $this->log(
                                (string) __(
                                    'Ignore order? %1',
                                    $ignoreAutomation ? __('Yes') : __('No')
                                )
                            );
                            if (
                                !$ignoreAutomation
                                && $order->canShip()
                            ) {
                                $this->log(
                                    (string) __(
                                        'Should automatically ship order %1..',
                                        $order->getId()
                                    )
                                );

                                $shipment = $this->orderConverter->toShipment($order);
                                $this->log(
                                    (string) __('Creating a new shipment')
                                );

                                // Loop through order items
                                $packageItems = [];
                                $totalWeight = 0;
                                $totalCustomsValue = 0;
                                $validOrderItems = 0;
                                foreach ($order->getAllItems() as $orderItem) {
                                    $weight = 0;

                                    // Check if order item has qty to ship or is virtual
                                    if (
                                        !$orderItem->getQtyToShip()
                                        || $orderItem->getIsVirtual()
                                    ) {
                                        continue;
                                    }
                                    $validOrderItems++;
                                    $qty = $orderItem->getQtyToShip();

                                    $customsValue =
                                        $orderItem->getRowTotal();
                                    $totalCustomsValue +=
                                        $customsValue;
                                    $weight =
                                        0;

                                    $product = null;
                                    try {
                                        $product = $this->productRepository->getById(
                                            $orderItem->getProductId()
                                        );
                                        if ($product) {
                                            $weight +=
                                                ($product->getWeight() * $orderItem->getQtyToShip());
                                        }
                                    } catch (\Exception $e) {
                                        $product = null;
                                    }
                                    $totalWeight += $weight;

                                    $packageItems[] = [
                                        'customs_value' => $customsValue,
                                        'name' => $orderItem->getName(),
                                        'order_item_id' => $orderItem->getItemId(),
                                        'price' => $orderItem->getPrice(),
                                        'product_id' => $orderItem->getProductId(),
                                        'qty' => $qty,
                                        'weight' => $weight,
                                    ];

                                    $qtyShipped = $orderItem->getQtyToShip();

                                    // Create shipment item with qty
                                    $shipmentItem = $this->orderConverter
                                        ->itemToShipmentItem($orderItem)->setQty($qtyShipped);

                                    // Add shipment item to shipment
                                    $shipment->addItem($shipmentItem);
                                }

                                $this->log(
                                    (string) __(
                                        'Added %1 valid order items to shipment',
                                        $validOrderItems
                                    )
                                );

                                if ($validOrderItems) {
                                    $packages = [
                                        'packages' => [
                                            [
                                                'items' => $packageItems,
                                                'params' => [
                                                    'container' => $this->metaHelper->getConfig(
                                                        'carriers/mspostnord/automation/package_type'
                                                    ),
                                                    'content_type' => '',
                                                    'content_type_other' => '',
                                                    'customs_value' => $totalCustomsValue,
                                                    'dimension_units' => 'METER',
                                                    'height' => '',
                                                    'length' => '',
                                                    'weight' => ($totalWeight ? $totalWeight : 1),
                                                    'weight_units' => 'KILOGRAM',
                                                    'width' => '',
                                                ],
                                            ]
                                        ]
                                    ];

                                    // Generate request here
                                    $request = new \Magento\Framework\App\Request\Http(
                                        $this->cookieReaderInterface,
                                        $this->stringUtils,
                                        $this->configInterfaceProxy,
                                        $this->pathInfoProcessorInterface,
                                        $this->objectManagerInterface
                                    );
                                    $request->setParams($packages);
                                    $this->log((string) __('Created shipment packages'));

                                    // Register shipment
                                    $shipment->register();
                                    $shipment->getOrder()->setIsInProcess(true);

                                    $adminUsername = $this->metaHelper->getConfig(
                                        'carriers/mspostnord/automation/username'
                                    );
                                    $oldUser = false;
                                    if (!empty($adminUsername)) {
                                        $this->log(
                                            (string) __(
                                                'Loading username "%1"..',
                                                $adminUsername
                                            )
                                        );
                                        $user = $this->userModel->loadByUsername($adminUsername);

                                        $this->log(
                                            (string) __(
                                                'Setting session to "%1"..',
                                                $adminUsername
                                            )
                                        );
                                        $oldUser = $this->authSession->getUser();
                                        $this->authSession->setUser($user);
                                    }

                                    try {
                                        // Create shipping label
                                        $this->log(
                                            (string) __('Creating shipping labels..')
                                        );
                                        $this->labelGenerator->create(
                                            $shipment,
                                            $request
                                        );
                                        $this->log(
                                            (string) __('Generated shipping labels')
                                        );

                                        // Revert session user again
                                        if ($oldUser) {
                                            $this->log(
                                                (string) __(
                                                    'Reverting session user back to "%1"..',
                                                    $oldUser->getUserName()
                                                )
                                            );
                                            $this->authSession->setUser($oldUser);
                                        }

                                        // TODO: refactor to a repository?
                                        // Save created shipment and order
                                        $shipment->save();
                                        $shipment->getOrder()->save();

                                        if (
                                            $this->metaHelper->getConfig(
                                                'carriers/mspostnord/automation/notify'
                                            )
                                        ) {
                                            $this->shipmentNotifier->notify($shipment);
                                            $this->log(
                                                (string) __('Sent notifications')
                                            );
                                        } else {
                                            $this->log(
                                                (string) __('Did not send notifications')
                                            );
                                        }

                                        $shipment->save();

                                        $this->log(
                                            (string) __(
                                                'Successfully shipped order %1',
                                                $order->getId()
                                            )
                                        );

                                        $successCount++;
                                    } catch (\Exception $e) {
                                        $this->log(
                                            (string) __(
                                                'Failed to ship order %1, error: "%2"',
                                                $order->getId(),
                                                __($e->getMessage())
                                            )
                                        );

                                        $orderData = $this->orderDataHelper->readData(
                                            (int) $order->getEntityId()
                                        );
                                        $orderData['ignoreAutomation'] = true;
                                        $this->orderDataHelper->update(
                                            (int) $order->getEntityId(),
                                            $orderData
                                        );
                                        $errorCount++;
                                    }
                                } else {
                                    $this->log(
                                        (string) __(
                                            'Skipping order since we found no valid order items'
                                        )
                                    );
                                }
                            } else {
                                $this->log(
                                    (string) __(
                                        'Ignoring order %1 because it cannot ship.',
                                        $order->getId()
                                    )
                                );
                            }
                        } else {
                            $this->log(
                                (string) __(
                                    'Skipping order %1 because it is using a fallback shipping-option',
                                    (int) $order->getEntityId()
                                )
                            );
                        }
                    }
                } else {
                    $this->log(
                        (string) __('Found no matching orders')
                    );
                }
            } else {
                $this->log(
                    (string) __('Automation disabled in store')
                );
            }

            // Get orders of last 14 days regardless of status to update status on
            $criteria = $this->searchCriteriaBuilder
                ->addFilter('shipping_method', 'mspostnord_%', 'like')
                    ->addFilter('store_id', $storeId)
                    ->addFilter('created_at', date('Y-m-d H:i:s', strtotime('-14 days')), 'gt')
                    ->create();
            if ($orders = $this->orderRepository->getList($criteria)->getItems()) {
                $limitPerBatch = 20;
                $ordersAndShipmentsBuffer = [];
                $ordersAndShipmentsBatches = [];
                foreach ($orders as $order) {
                    /**
                     * If we have a shipment-id
                     * store in queue to update statuses
                     */
                    $orderData = $this->orderDataHelper->readData(
                        (int) $order->getEntityId()
                    );
                    if (
                        !empty($orderData['shipments'])
                        && is_array($orderData['shipments'])
                    ) {
                        foreach ($orderData['shipments'] as $shipment) {
                            if (
                                !empty($shipment['id'])
                                && is_string($shipment['id'])
                            ) {
                                $ordersAndShipmentsBuffer[] = [
                                    (int) $order->getEntityId(),
                                    $shipment['id']
                                ];
                                if (count($ordersAndShipmentsBuffer) === $limitPerBatch) {
                                    $ordersAndShipmentsBatches[] =
                                        $ordersAndShipmentsBuffer;
                                    $ordersAndShipmentsBuffer = [];
                                }
                            }
                        }
                    }
                }
                if ($ordersAndShipmentsBuffer) {
                    $ordersAndShipmentsBatches[] =
                        $ordersAndShipmentsBuffer;
                    $ordersAndShipmentsBuffer = [];
                }

                if ($ordersAndShipmentsBatches) {
                    foreach ($ordersAndShipmentsBatches as $ordersAndShipments) {
                        $this->shipmentHelper->shipmentStatuses(
                            $ordersAndShipments,
                            true
                        );
                        $this->log(
                            (string) __(
                                'Refreshed %1 shipment statuses in store',
                                count($ordersAndShipments)
                            )
                        );
                    }
                } else {
                    $this->log(
                        (string) __('Found no shipments in store to refresh status on')
                    );
                }
            }
        }
        $this->log(
            sprintf(
                (string) __('Automation finished, successes: %d, failures: %d'),
                $successCount,
                $errorCount
            )
        );

        $sortOrder =
            $this->sortOrderBuilder->setField('id')->setDirection('ASC')->create();

        // Get 15 oldest order data, if we can't find referenced order delete it
        $searchCriteria = $this->searchCriteriaBuilder
            ->create()->setSortOrders([$sortOrder])->setPageSize(15);
        if (
            $orderData = $this->orderDataRepositoryInterface->getList($searchCriteria)->getItems()
        ) {
            foreach ($orderData as $orderDatum) {
                $orderId =
                    $orderDatum->getOrderId();
                try {
                    $this->orderRepository->get($orderId);
                } catch (\Magento\Framework\Exception\NoSuchEntityException $e) {
                    $this->orderDataHelper->delete(
                        $orderId
                    );
                    $this->log(
                        (string) __(
                            'Deleted order-data for missing order %1',
                            $orderId
                        )
                    );
                }
            }
        } else {
            $this->log(
                (string) __(
                    'Found no order data'
                )
            );
        }

        // Get 15 oldest persistent quote data, if we can't find referenced quote delete it
        $searchCriteria = $this->searchCriteriaBuilder
            ->create()->setSortOrders([$sortOrder])->setPageSize(15);
        if ($quotes = $this->persistentQuoteRepositoryInterface->getList($searchCriteria)->getItems()) {
            foreach ($quotes as $quote) {
                $quoteId =
                    $quote->getQuoteId();
                try {
                    $this->cartRepositoryInterface->get($quoteId);
                } catch (\Magento\Framework\Exception\NoSuchEntityException $e) {
                    $this->persistentQuoteHelper->delete(
                        $quoteId
                    );
                    $this->log(
                        (string) __(
                            'Deleted persistent quote data for missing quote %1',
                            $quoteId
                        )
                    );
                }
            }
        } else {
            $this->log(
                (string) __(
                    'Found no persistent quote data'
                )
            );
        }

        $status = $errorCount
            ? \Mediastrategi\PostNord\Api\Data\ScheduleInterface::ERROR
            : \Mediastrategi\PostNord\Api\Data\ScheduleInterface::OK;
        if (!($successCount === 0 && $errorCount === 0)) {
            $this->saveSchedule($status);
        }

        $this->cleanup();
    }

    private function log(string $message)
    {
        $line = date('Y-m-d H:i:s') . ' | ' . $message . "\n";
        $this->log .= $line;
    }

    private function saveSchedule(
        $status = \Mediastrategi\PostNord\Api\Data\ScheduleInterface::OK
    ) {
        $data = [
            \Mediastrategi\PostNord\Api\Data\ScheduleInterface::SCHEDULE_STATUS =>
                $status,
            \Mediastrategi\PostNord\Api\Data\ScheduleInterface::SCHEDULE_LOG =>
                $this->getLog(),
        ];

        if (!empty($data)) {
            $this->scheduleHelper->add($data);
        }
    }

    private function cleanup(): void
    {
        $this->apiTransactionHelper->clean();
        $this->scheduleHelper->clean();
    }
}
