/** @see Magento_Checkout/js/view/cart/shipping-rates */
/* global define */
define([
    'jquery',
    'ko',
    'mage/utils/wrapper',
    'Magento_Checkout/js/model/quote',
    'Magento_Checkout/js/action/select-shipping-address',
    'Magento_Checkout/js/model/new-customer-address',
    'Magento_Checkout/js/model/shipping-service',
    'Magento_Checkout/js/action/set-shipping-information',
    'require'
], function (
    $,
    ko,
    wrapper,
    quoteModel,
    selectShippingAddress,
    newCustomerAddress,
    shippingService,
    setShippingInformationAction,
    require
) {
    'use strict';

    var mixin = {
        debug: function(callback) {
            if (window.mspostnord_debug) {
                console.log('Mediastrategi PostNord Ship Connect Debug Start');
                callback();
                console.log('Mediastrategi PostNord Ship Connect Debug End');
            }
        },
        ajaxInProgress: null,
        state: {},
        stateChangeGracePeriod: 250,
        stateChangeTimer: null,
        initialize: function()
        {
            this._super();
            var parent = this;
            this.debug(function() {
                console.log('shipping-rates.js initialize()');
                console.log(parent);
            });
            this.refreshRatesStack = 0;
            this.setInitial = true;
            window.mspostnord_state = {};

            // When rates changes, decoded agents and add-on if it exists
            if (typeof(this.shippingRates) === 'undefined') {
                this.shippingRates = ko.observableArray();
            }
            this.shippingRates.subscribe(function(rates) {
                parent.refreshRates(rates, parent);
            });

            // Initialize setting of selected method
            this.selectedMethod = ko.observable(
                this.selectedShippingMethod()
            );

            // When changing selected-method in our template, update checkout shipping method
            this.selectedMethod.subscribe(function(selectedRate) {
                parent.debug(function() {
                    console.log('selected method changed to: ' + selectedRate);
                });
                if (typeof(parent) !== 'undefined'
                    && typeof(parent.shippingRates) !== 'undefined'
                   ) {
                    var rates = parent.shippingRates();
                    var found_rate = false;
                    for (var rateIndex in rates)
                    {
                        if (rates.hasOwnProperty(rateIndex)) {
                            var rate = rates[rateIndex];
                            if (typeof(rate.carrier_code) !== 'undefined'
                                && typeof(rate.method_code) !== 'undefined'
                                && rate.carrier_code + '_' + rate.method_code == selectedRate
                               ) {
                                found_rate = true;
                                parent.selectShippingMethod(rate);
                                break;
                            }
                        }
                    }
                }
            });

            this.allOptionsAreFromPostNord = ko.observable(false);

            // When selected shipping method changes externally, select in internally as well
            quoteModel.shippingMethod.subscribe(function(method) {
                if (
                    typeof(method) === 'object'
                    && method !== null
                    && method.hasOwnProperty('carrier_code')
                    && method.hasOwnProperty('method_code')
                ) {
                    var methodId =
                        method['carrier_code'] + '_' + method['method_code'];
                    if (parent.selectedMethod() !== methodId) {
                        parent.selectedMethod(method['carrier_code'] + '_' + method['method_code']);
                    }
                }
            });

            // Initial run
            parent.refreshRates(
                this.shippingRates(),
                parent
            );

            return this;
        },
        toggleTooltip: function (element, method, rates, _data, event) {
            event.preventDefault();
            event.stopImmediatePropagation();
            var isMobile =
                ($(window).width() <= 540);
            for (var rateIndex in rates) {
                if (rates.hasOwnProperty(rateIndex)) {
                    var anotherRate =
                        rates[rateIndex];
                    if (anotherRate !== method) {
                        if (typeof(anotherRate.mspostnord_show_tooltip) !== 'undefined') {
                            anotherRate.mspostnord_show_tooltip(false);
                        }
                    }
                }
            }

            if (method.mspostnord_show_tooltip() === false) {
                if (isMobile) {
                    $('.tooltip-wrapper', $(element).parent().parent()).first().css('top', '').css('left', '');
                } else {
                    // Adjust tooltip position
                    var tooltip =
                        $('.tooltip-wrapper', $(element).parent().parent()).first();
                    var newTop =
                        22 - 1 * Math.round(tooltip.outerHeight() / 2);
                    var newLeft =
                        80 + element.offsetLeft;
                    $(tooltip).css('top', newTop);
                    $(tooltip).css('left', newLeft);

                    // Adjust tooltip pointers position
                    var tooltipPointer1 =
                        $('.tooltip-pointer-1', $(element).parent().parent()).first();
                    var tooltipPointer2 =
                        $('.tooltip-pointer-2', $(element).parent().parent()).first();
                    var newTop1 =
                        Math.round(tooltip.outerHeight() / 2) - 6;
                    $(tooltipPointer1).css('top', newTop1);
                    $(tooltipPointer2).css('top', newTop1 - 1);
                }
            }

            method.mspostnord_show_tooltip(
                !method.mspostnord_show_tooltip()
            );
        },
        refreshRates: function (rates, parent)
        {
            this.refreshRatesStack++;
            this.debug(function() {
                console.log('refreshRates() - starting ' + parent.refreshRatesStack);
            });
            var allOptionsAreFromPostNord = false;
            if (rates.length) {
                this.debug(function() {
                    console.log('New rates before process: ');
                    var ratesDeepCopy = $.extend(true, {}, rates);
                    console.log(ratesDeepCopy);
                });

                var rateIndex, rate, decoded, updated, shipmentOptionId;
                allOptionsAreFromPostNord = true;
                updated = false;
                for (rateIndex in rates) {
                    if (rates.hasOwnProperty(rateIndex)) {
                        rate = rates[rateIndex];
                        if (rate.hasOwnProperty('extension_attributes')) {

                            // Show tooltip flag
                            if (!rate.hasOwnProperty('mspostnord_show_tooltip')) {
                                rate.mspostnord_show_tooltip = ko.observable(false);
                                updated = true;
                            }

                            // Shipment Option Id
                            if (rate.extension_attributes.hasOwnProperty('mspostnord_shipment_option_id')
                                && !rate.hasOwnProperty('mspostnord_shipment_option_id')
                               ) {
                                rate.mspostnord_shipment_option_id =
                                    rate.extension_attributes.mspostnord_shipment_option_id;
                                updated = true;
                            }

                            // Is Recommended
                            if (rate.extension_attributes.hasOwnProperty('mspostnord_is_recommended')
                                && !rate.hasOwnProperty('mspostnord_is_recommended')
                               ) {
                                rate.mspostnord_is_recommended =
                                    rate.extension_attributes.mspostnord_is_recommended;
                                updated = true;
                            }

                            // ETA
                            if (rate.extension_attributes.hasOwnProperty('mspostnord_delivery_estimate_text')
                                && !rate.hasOwnProperty('mspostnord_delivery_estimate_text')
                               ) {
                                rate.mspostnord_delivery_estimate_text =
                                    rate.extension_attributes.mspostnord_delivery_estimate_text;
                                updated = true;
                            }

                            shipmentOptionId =
                                rate.mspostnord_shipment_option_id
                                ? rate.mspostnord_shipment_option_id : '';
                            if (shipmentOptionId) {
                                if (typeof(parent.state[shipmentOptionId]) === 'undefined') {
                                    parent.state[shipmentOptionId] = {
                                        carrier_title: rate.carrier_title,
                                        method_title: rate.method_title,
                                        selectedAddons: {},
                                        selectedServicePoint: {}
                                    };
                                }

                                var servicePointIdToArrayIndex = {};
                                var i;
                                var servicePointLength, servicePoint, servicePointId, servicePointIndex;
                                var addonLength, addon, addonId, addonIndex, addonValue;

                                // Available service-points
                                if (rate.extension_attributes.hasOwnProperty('mspostnord_available_service_points')
                                    && !rate.hasOwnProperty('mspostnord_available_service_points')
                                   ) {
                                    var availableServicePoints = [];
                                    try {
                                        decoded = $.parseJSON(
                                            rate.extension_attributes.mspostnord_available_service_points
                                        );
                                    } catch (e) {
                                        decoded = false;
                                        console.error("parseJSON error", e);
                                    }
                                    if (typeof(decoded) === 'object' &&
                                        decoded.length
                                       ) {
                                        /**
                                         * Store reference to shipmentOptionId on each service-point
                                         * for quick access to state data
                                         */
                                        servicePointLength = decoded.length;
                                        for (i = 0; i < servicePointLength; i++) {
                                            servicePoint = decoded[i];
                                            if (typeof(servicePoint) === 'object'
                                                && servicePoint.hasOwnProperty('servicePointId')
                                               ) {
                                                servicePointId =
                                                    servicePoint.servicePointId;
                                                servicePointIndex =
                                                    availableServicePoints.length;
                                                servicePoint.shipmentOptionId =
                                                    shipmentOptionId;
                                                availableServicePoints.push(servicePoint);
                                                servicePointIdToArrayIndex[servicePointId] =
                                                    servicePointIndex;
                                            }
                                        }
                                    }
                                    rate.mspostnord_available_service_points =
                                        ko.observableArray(availableServicePoints);
                                    updated = true;
                                }

                                // Selected service-point
                                if (rate.extension_attributes.hasOwnProperty('mspostnord_selected_service_point')
                                    && !rate.hasOwnProperty('mspostnord_selected_service_point')
                                   ) {
                                    var selectedServicePoint = {};
                                    try {
                                        decoded = $.parseJSON(
                                            rate.extension_attributes.mspostnord_selected_service_point
                                        );
                                    } catch (e) {
                                        decoded = false;
                                        console.error("parseJSON error", e);
                                    }
                                    parent.debug(function() {
                                        console.log('decoded selected agent:');
                                        console.log(decoded);
                                    });
                                    if (typeof(decoded) === 'object' &&
                                        Object.keys(decoded).length &&
                                        decoded.hasOwnProperty('servicePointId') &&
                                        rate.hasOwnProperty('mspostnord_available_service_points')
                                       ) {
                                        servicePointId =
                                            decoded.servicePointId;
                                        if (typeof(
                                            servicePointIdToArrayIndex[servicePointId]
                                        ) !== 'undefined') {
                                            servicePointIndex =
                                                servicePointIdToArrayIndex[servicePointId];
                                            selectedServicePoint =
                                                rate.mspostnord_available_service_points()[servicePointIndex];
                                            parent.debug(function() {
                                                console.log('saved selected agent');
                                            });
                                        }
                                    }
                                    rate.mspostnord_selected_service_point =
                                        ko.observable(selectedServicePoint);
                                    parent.state[shipmentOptionId]["selectedServicePoint"] =
                                        selectedServicePoint;

                                    // When changing selected service-point, update window state
                                    rate.mspostnord_selected_service_point.subscribe(function(newAgent) {
                                        parent.debug(function() {
                                            console.log('newAgent: ');
                                            console.log(newAgent);
                                        });
                                        parent.stateUpdate();
                                    });
                                    updated = true;
                                }

                                // Available add-ons
                                if (rate.extension_attributes.hasOwnProperty('mspostnord_available_addons')
                                    && !rate.hasOwnProperty('mspostnord_available_addons')
                                   ) {
                                    var availableAddons = {};
                                    try {
                                        decoded = $.parseJSON(
                                            rate.extension_attributes.mspostnord_available_addons
                                        );
                                    } catch (e) {
                                        decoded = false;
                                        console.error("parseJSON error", e);
                                    }
                                    if (typeof(decoded) === 'object' &&
                                        decoded.length
                                       ) {
                                        /**
                                         * Store reference to shipmentOptionId on each add-on
                                         * for quick access to state data
                                         */
                                        addonLength = decoded.length;
                                        for (i = 0; i < addonLength; i++) {
                                            addon = decoded[i];
                                            if (typeof(addon) === 'object'
                                                && addon.hasOwnProperty('id')
                                               ) {
                                                addon.shipmentOptionId =
                                                    shipmentOptionId;
                                                availableAddons[addon.id] =
                                                    addon;
                                            }
                                        }
                                    }
                                    rate.mspostnord_available_addons =
                                        availableAddons;
                                    updated = true;
                                }

                                // Selected add-ons
                                if (rate.extension_attributes.hasOwnProperty('mspostnord_selected_addons')
                                    && !rate.hasOwnProperty('mspostnord_selected_addons')
                                    && rate.hasOwnProperty('mspostnord_available_addons')
                                   ) {
                                    var selectedAddons = {};
                                    try {
                                        decoded = $.parseJSON(
                                            rate.extension_attributes.mspostnord_selected_addons
                                        );
                                    } catch (e) {
                                        decoded = false;
                                        console.error("parseJSON error", e);
                                    }
                                    parent.debug(function() {
                                        console.log('decoded selected add-ons:');
                                        console.log(decoded);
                                    });
                                    if (typeof(decoded) === 'object' &&
                                        Object.keys(decoded).length
                                       ) {
                                        // Only keep value part here
                                        var newSelectedAddons = {};
                                        for (var selectedAddonId in decoded) {
                                            if (decoded.hasOwnProperty(selectedAddonId)) {
                                                var selectedAddon =
                                                    decoded[selectedAddonId];
                                                if (selectedAddon.hasOwnProperty('value')) {
                                                    newSelectedAddons[selectedAddonId] =
                                                        selectedAddon.value;

                                                    // Convert boolean strings to booleans
                                                    if (newSelectedAddons[selectedAddonId] === 'true') {
                                                        newSelectedAddons[selectedAddonId] = true;
                                                    } else if (newSelectedAddons[selectedAddonId] === 'false') {
                                                        newSelectedAddons[selectedAddonId] = false;
                                                    }
                                                }
                                            }
                                        }
                                        if (Object.keys(newSelectedAddons).length) {
                                            selectedAddons = newSelectedAddons;
                                            parent.debug(function() {
                                                console.log('saved selected add-ons');
                                                console.log(newSelectedAddons);
                                            });
                                        }
                                    }

                                    // Allocate values for all available add-ons that are not selected
                                    for (addonId in rate.mspostnord_available_addons) {
                                        if (rate.mspostnord_available_addons.hasOwnProperty(addonId)) {
                                            addon =
                                                rate.mspostnord_available_addons[addonId];
                                            if (typeof(addon.type) !== 'undefined'
                                                && typeof(selectedAddons[addonId]) === 'undefined'
                                               ) {
                                                if (addon.type === 'checkbox') {
                                                    selectedAddons[addonId] = false;
                                                } else if (addon.type === 'doorCode') {
                                                    selectedAddons[addonId] = '';
                                                }
                                            }
                                        }
                                    }

                                    rate.mspostnord_selected_addons =
                                        selectedAddons;
                                    parent.state[shipmentOptionId]["selectedAddons"] =
                                        selectedAddons;
                                    updated = true;
                                }

                                // Build observable array for easier presentation of add-ons
                                if (rate.hasOwnProperty('mspostnord_selected_addons')
                                    && rate.hasOwnProperty('mspostnord_available_addons')
                                    && !rate.hasOwnProperty('mspostnord_addons')
                                   ) {
                                    var addons = [];
                                    for (addonId in rate.mspostnord_available_addons) {
                                        if (rate.mspostnord_available_addons.hasOwnProperty(addonId)) {
                                            addon =
                                                rate.mspostnord_available_addons[addonId];
                                            addonValue =
                                                rate.mspostnord_selected_addons[addonId];
                                            var newAddon = {
                                                helpText: addon.helpText,
                                                id: addon.id,
                                                name: addon.name,
                                                shipmentOptionId: addon.shipmentOptionId,
                                                type: addon.type,
                                                value: ko.observable(addonValue)
                                            };
                                            newAddon.value.subscribe(function(newAddonValue) {
                                                parent.debug(function() {
                                                    console.log('newAddonValue:');
                                                    console.log(newAddonValue);
                                                });
                                                parent.stateUpdate();
                                            });
                                            addons.push(newAddon);
                                        }
                                    }
                                    rate.mspostnord_addons =
                                        ko.observableArray(addons);
                                    updated = true;
                                }
                            } else {
                                allOptionsAreFromPostNord = false;
                            }
                        } else {
                            allOptionsAreFromPostNord = false;
                        }
                    }
                }
                if (updated) {
                    shippingService.setShippingRates(rates);
                }
            }

            parent.debug(function() {
                console.log('All options were from PostNord: ' + allOptionsAreFromPostNord);
            });
            this.allOptionsAreFromPostNord(allOptionsAreFromPostNord);
            this.refreshRatesStack--;
            if (rates.length &&
                !this.setInitial &&
                this.refreshRatesStack === 0
               ) {
                if (typeof(this.isSelected) !== 'undefined') {
                    this.selectedMethod(this.isSelected());
                }
                this.setInitial = true;
            }
        },
        saveRate: function(rate) {
            this.selectShippingMethod(rate);

            // Klarna Checkout support
            if (require.defined('Klarna_Kco/js/model/kco')) {
                require('Klarna_Kco/js/model/kco').refreshAddresses();
                setShippingInformationAction();
            }

            // Svea Checkout support
            if (typeof(window.scoApi) !== 'undefined') {
                // NOTE It seems this resolves the issue with Svea Checkout not updating prices
                if (require.defined('Magento_Checkout/js/view/cart/shipping-estimation')) {
                    require('Magento_Checkout/js/view/cart/shipping-estimation')([]);
                }
            }
        },
        stateUpdate: function ()
        {
            var parent = this;
            parent.debug(function() {
                console.log('stateUpdate');
            });
            if (parent.mspostnord_state_update_timer !== null) {
                clearTimeout(parent.mspostnord_state_update_timer);
            }
            parent.mspostnord_state_update_timer = setTimeout(
                function() {
                    var state =
                        {};
                    var rates =
                        shippingService.getShippingRates()();
                    parent.debug(function() {
                        console.log('update state from rates:');
                        console.log(rates);
                    });
                    var rateIndex, rateCount, rate, shipmentOptionId;
                    rateCount = rates.length;
                    for (rateIndex = 0; rateIndex < rateCount; rateIndex++) {
                        rate = rates[rateIndex];
                        if (typeof(rate.mspostnord_shipment_option_id) !== 'undefined') {
                            shipmentOptionId =
                                rate.mspostnord_shipment_option_id;
                            state[shipmentOptionId] = {
                                'carrier_title': rate.carrier_title,
                                'method_title': rate.method_title,
                                'selectedAddons': {},
                                'selectedServicePoint': {}
                            };

                            // Selected service-point
                            if (typeof(rate.mspostnord_selected_service_point) !== 'undefined') {
                                state[shipmentOptionId]['selectedServicePoint'] =
                                    rate.mspostnord_selected_service_point();
                            }

                            // Selected add-ons
                            if (typeof(rate.mspostnord_addons) !== 'undefined') {
                                var addonIndex, addonCount, addon;
                                addonCount = rate.mspostnord_addons().length;
                                for (addonIndex = 0; addonIndex < addonCount; addonIndex++) {
                                    addon =
                                        rate.mspostnord_addons()[addonIndex];
                                    var addonId =
                                        addon.id;
                                    var addonValue =
                                        addon.value();
                                    var isSelected =
                                        addonValue !== false
                                        && addonValue !== '';
                                    state[shipmentOptionId]['selectedAddons'][addonId] = {
                                        name: addon.name,
                                        helpText: addon.helpText,
                                        type: addon.type,
                                        value: addonValue
                                    };
                                }
                            }
                        }
                    }

                    window.mspostnord_state = state;

                    // Cancel any ongoing XHR-request
                    if (parent.ajaxInProgress) {
                        parent.ajaxInProgress.abort();
                    }

                    parent.ajaxInProgress = $.ajax({
                        data: {
                            state: state
                        },
                        dataType: 'json',
                        error: function(response) {
                            parent.debug(function() {
                                console.log('ajax error response:');
                                console.log(response);
                            });
                        },
                        success: function(response) {
                            parent.debug(function() {
                                console.log('ajax success response:');
                                console.log(response);
                            });
                        },
                        type: 'post',
                        url: window.mspostnord_checkout_url
                    });

                    parent.debug(function() {
                        console.log('new state:');
                        console.log(window.mspostnord_state);
                    });
                },
                parent.mspostnord_state_grace_period
            );
        }
    };
    return function (target) {
        target = target.extend(mixin);

        if (typeof(target.defaults.template) !== 'undefined') {
            target.defaults.template =
                'Mediastrategi_PostNord/cart/shipping-rates';
        }

        $('body').addClass('mspostnord-loaded');
        return target;
    };
});
